## Christopher Potts and Florian Schwarz
## For "Affective 'this'" (Linguistic Issues in Language Technology)
## Submitted 2009-06-19
## Revised 2009-12-11
##
## The function solidarityPlot() produces the plots and model
## summaries for the solidarity experiment.
##
## Arguments:
##  -- Df: data.frame (either byproduct.english.dems.csv or byproduct.german.dies.csv)
##  -- Tokencolumn: a column of 'successes' (e.g. Df$thisDET or Df$dies).
##  -- Title: A title for the plot. Defaults to no title ("").
##
##  Output/side-effects:
##  -- A plot window with regression line in red.
##  -- A model summary (to the screen).
solidarityPlot = function(Df, Tokencolumn, Title=""){
	
  ## Fit the model.
  fit = glm(formula = cbind(Tokencolumn, Words - Tokencolumn) ~ PercentageOfReviews, family="binomial", data=Df)
  
  print(summary(fit)) 

  ## Get and round the coefficient.
  roundedCoef = round(coef(fit)[2], 3)
	
  ## Process the p value so that it doesn't ever print, even after rounding.
  printableP = as.data.frame(summary(fit)[12])[2,4]
  if (printableP < 0.001){
    printableP = "p < 0.001"	
  }
  else {
    printableP	= paste("p =", round(printableP, 3), sep=" ")
  }
	
  ## Add log-odds.
  Df$logodds = log(Tokencolumn/(Df$Words - Tokencolumn))	
	
  ## Trim errant values.
  Df = subset(Df, Df$logodds <= 0.0 & Df$logodds >= -100000)
  
  ## Remove outliers from the plot by limiting the view to within 3 s.d. of the mean.
  logodds.sd = sd(Df$logodds)
  logodds.mean = mean(Df$logodds)
  ymin = logodds.mean + (logodds.sd * 3)
  ymax = logodds.mean - (logodds.sd * 3)
	
  ## Plot.
  plot(Df$PercentageOfReviews, Df$logodds, 
       type="p", cex=0.4, col="darkgray", 
       xlab="Pr(Product, Rating)", 
       ylab="Log-odds", 
       main=Title,       
       ylim=c(ymax,ymin),
       sub=paste("coef = ", roundedCoef, " (", printableP, ")", sep=""))
  
  ## Add the logistic curve, staying in log-odds space (no plogis/invlogit).
  curve(coef(fit)[1] + coef(fit)[2]*x, add=TRUE, col="red", lwd=2)
}

